@@ -11,32 +11,18 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 from botocore.model import ServiceModel
-from botocore.exceptions import ParamValidationError
 from botocore.exceptions import DataNotFoundError
 from botocore.exceptions import OperationNotPageableError
+from botocore.exceptions import ClientError
 from botocore import waiter
 from botocore import xform_name
 from botocore.paginate import Paginator
-from botocore import translate
+from botocore.utils import CachedProperty
 import botocore.validate
 import botocore.serialize
 from botocore import credentials
 
 
-class ClientError(Exception):
-    MSG_TEMPLATE = (
-        'An error occurred ({error_code}) when calling the {operation_name} '
-        'operation: {error_message}')
-
-    def __init__(self, error_response, operation_name):
-        msg = self.MSG_TEMPLATE.format(
-            error_code=error_response['Error']['Code'],
-            error_message=error_response['Error']['Message'],
-            operation_name=operation_name)
-        super(ClientError, self).__init__(msg)
-        self.response = error_response
-
-
 class ClientCreator(object):
     """Creates client objects for a service."""
     def __init__(self, loader, endpoint_creator, event_emitter):
@@ -118,7 +104,7 @@ class ClientCreator(object):
             if 'page_config' not in self._cache:
                 try:
                     page_config = loader.load_data('aws/%s/%s.paginators' % (
-                        service_model.endpoint_prefix,
+                        service_model.service_name,
                         service_model.api_version))['pagination']
                     self._cache['page_config'] = page_config
                 except DataNotFoundError:
@@ -138,41 +124,45 @@ class ClientCreator(object):
             if 'waiter_config' not in self._cache:
                 try:
                     waiter_config = loader.load_data('aws/%s/%s.waiters' % (
-                        service_model.endpoint_prefix,
-                        service_model.api_version))['waiters']
-                    self._cache['waiter_config'] = translate.denormalize_waiters(
-                        waiter_config)
+                        service_model.service_name,
+                        service_model.api_version))
+                    self._cache['waiter_config'] = waiter_config
                 except DataNotFoundError:
                     self._cache['waiter_config'] = {}
             return self._cache['waiter_config']
 
         def get_waiter(self, waiter_name):
             config = self._get_waiter_config()
+            if not config:
+                raise ValueError("Waiter does not exist: %s" % waiter_name)
+            model = waiter.WaiterModel(config)
             mapping = {}
-            for name in config:
+            for name in model.waiter_names:
                 mapping[xform_name(name)] = name
             if waiter_name not in mapping:
                 raise ValueError("Waiter does not exist: %s" % waiter_name)
-            single_waiter_config = config[mapping[waiter_name]]
-            return waiter.Waiter(
-                waiter_name,
-                getattr(self, xform_name(single_waiter_config['operation'])),
-                single_waiter_config)
 
-        def all_waiters(self):
+            return waiter.create_waiter_with_client(
+                mapping[waiter_name], model, self)
+
+        @CachedProperty
+        def waiter_names(self):
             """Returns a list of all available waiters."""
-            all_waiters = self._get_waiter_config()
+            config = self._get_waiter_config()
+            if not config:
+                return[]
+            model = waiter.WaiterModel(config)
             # Waiter configs is a dict, we just want the waiter names
             # which are the keys in the dict.
-            return [xform_name(name) for name in all_waiters]
+            return [xform_name(name) for name in model.waiter_names]
 
         methods_dict['_get_waiter_config'] = _get_waiter_config
         methods_dict['get_waiter'] = get_waiter
-        methods_dict['all_waiters'] = all_waiters
+        methods_dict['waiter_names'] = waiter_names
 
     def _load_service_model(self, service_name):
         json_model = self._loader.load_service_model('aws/%s' % service_name)
-        service_model = ServiceModel(json_model)
+        service_model = ServiceModel(json_model, service_name=service_name)
         return service_model
 
     def _get_client_args(self, service_model, region_name, is_secure,
@@ -226,7 +216,7 @@ class ClientCreator(object):
         def _api_call(self, **kwargs):
             operation_model = service_model.operation_model(operation_name)
             self._event_emitter.emit(
-                'before-parameter-build.{endpoint_prefix}.{operation_name}'\
+                'before-parameter-build.{endpoint_prefix}.{operation_name}'
                     .format(endpoint_prefix=service_model.endpoint_prefix,
                             operation_name=operation_name),
                 params=kwargs, model=operation_model)
@@ -271,3 +261,29 @@ class BaseClient(object):
         self._response_parser = response_parser
         self._event_emitter = event_emitter
         self._cache = {}
+
+    def clone_client(self, serializer=None, endpoint=None,
+                     response_parser=None, event_emitter=None):
+        """Create a copy of the client object.
+
+        All the parameters mirror the same arguments accepted by the
+        ``__init__`` of the client object.  Any argument not provided
+        will default to the value associated with the current client
+        object.  For example, if no ``endpoint`` argument is not provided,
+        then ``self._endpoint`` will be used.  This allows you to
+        either create an exact clone of the client, or create a clone with
+        certain objects modified.
+
+        :return: A new copy of the botocore client.
+
+        """
+        kwargs = {
+            'serializer': serializer,
+            'endpoint': endpoint,
+            'response_parser': response_parser,
+            'event_emitter': event_emitter,
+        }
+        for key, value in kwargs.items():
+            if value is None:
+                kwargs[key] = getattr(self, '_%s' % key)
+        return self.__class__(**kwargs)
